package badges;


import java.util.List;
import file.handling.ReadXMLFile;
import file.handling.TaskProperties;
import summary.metric.extraction.MetricsSummary;

public class BadgeCollection {
	private MetricsSummary studentMetricsSummary;
	private MetricsSummary solutionMetricsSummary;
	private ReadXMLFile xmlProperties;
	private String taskName;
	private List<String> relevantBadgeList;
	
	public ConditionalStatementBadge conditionalBadge;
	public GeneralMetricsBadge generalMetricsBadge;
	public IterationMetricsBadge iterationMetricsBadge;
	public OverallMetricsBadge overallMetricsBadge;
	public ExceptionMetricsBadge exceptionMetricsBadge;
	public InheritanceMetricsBadge inheritanceMetricsBadge;
	
	public BadgeCollection(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, ReadXMLFile xmlProperties, String taskName) {
		this.studentMetricsSummary = studentMetricsSummary;
		this.solutionMetricsSummary = solutionMetricsSummary;
		this.xmlProperties = xmlProperties;
		this.taskName = taskName;
		createRelevantBadgeList();
		createBadges();
	}
	
	private void createRelevantBadgeList() {
		for(TaskProperties taskProprty: xmlProperties.taskProperties) {
			if(taskProprty.id.equals(taskName)) {
				relevantBadgeList = taskProprty.badges;
			}	
		}
	}
	
	public void createBadges() {	
        
		for (String metric: relevantBadgeList) {
			System.out.println(metric);
			if(metric.equals("ConditionalStatementMetrics"))
				this.conditionalBadge = new ConditionalStatementBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.conditionalStatementMetricsScoreValues);
			
			else if(metric.equals("GeneralMetrics"))
				this.generalMetricsBadge = new GeneralMetricsBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.generalMetricsScoreValues);
			
			else if(metric.equals("IterationMetrics"))
				this.iterationMetricsBadge = new IterationMetricsBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.iterationMetricsScoreValues);
			
			else if(metric.equals("ExceptionMetrics"))
				this.exceptionMetricsBadge = new ExceptionMetricsBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.exeptionMetricsScoreValues);
			
			else if(metric.equals("OverallMetrics")) {
				this.overallMetricsBadge = new OverallMetricsBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.overallMetricsScoreValues);
				System.out.println("OOMB "+ overallMetricsBadge);
			}
			
			else if(metric.equals("InheritanceMetrics")) 
				this.inheritanceMetricsBadge = new InheritanceMetricsBadge(studentMetricsSummary, solutionMetricsSummary, xmlProperties.inherhitanceMetricsScoreValues);	
		}
		
	}
	
	private String printIfNotNull(Object o) {
		if (o==null) {
			return "";			
		}
		else {
			return o.toString();
		}
	}
	
	
	public String toString() {
		return "\n\nBadge collection\n"+
				printIfNotNull(conditionalBadge) +
				printIfNotNull(generalMetricsBadge)+ 
				printIfNotNull(iterationMetricsBadge) + 
				printIfNotNull(exceptionMetricsBadge)+
				printIfNotNull(overallMetricsBadge)+
				printIfNotNull(inheritanceMetricsBadge);
	}
}
