package badges;

import java.text.DecimalFormat;
import java.util.List;

import summary.metric.extraction.ConditionalStatementsSummary;
import summary.metric.extraction.MetricsSummary;

public class ConditionalStatementBadge {
	private ConditionalStatementsSummary studentMetrics;
	private ConditionalStatementsSummary solutionMetrics;
	private List<Double> conditionalStatementMedalValues;
	private double totalRatio = 0;
	private double numberOfValidRatios = 0;
	public Valor valor;
	public double numberOfIfsRatio;
	public double operatorsUsedRatio;
	public double cyclomaticComplexityRatio;
	public double cyclomaticComplexity;
	
	public ConditionalStatementBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> conditionalStatementMedalValues) {
		this.studentMetrics = studentMetricsSummary.conditionalStatementsMetrics;
		this.solutionMetrics = solutionMetricsSummary.conditionalStatementsMetrics;
		this.conditionalStatementMedalValues = conditionalStatementMedalValues;
		compareStudentMetrics();
		getValor();
		System.out.println("Cyclomatic Complexity : " + cyclomaticComplexity);
	}
	
	private void getValor() {


		if(!Double.isNaN(numberOfIfsRatio)) {
			totalRatio +=numberOfIfsRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(operatorsUsedRatio)) {
			totalRatio +=operatorsUsedRatio;
			numberOfValidRatios ++;
		}		
		if(!Double.isNaN(cyclomaticComplexityRatio)) {
			totalRatio +=cyclomaticComplexityRatio;
			numberOfValidRatios ++;
		}
		
		if (totalRatio/numberOfValidRatios <= conditionalStatementMedalValues.get(0))
			valor=Valor.GOLD;
		else if(totalRatio/numberOfValidRatios <= conditionalStatementMedalValues.get(1))
			valor=Valor.SILVER;
		else if(totalRatio/numberOfValidRatios <= conditionalStatementMedalValues.get(2))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}		
	}
	
	private void compareNumberOfIfs() {
		numberOfIfsRatio = (double)studentMetrics.numberOfIfs/(double)solutionMetrics.numberOfIfs;
	};

	private void compareStudentMetrics() {
		compareNumberOfIfs();
		calculateOperatorsUsedRatio();
		calculateCyclomaticComplexityRatio();
		
	}
	
	private void calculateCyclomaticComplexityRatio() {
		cyclomaticComplexityRatio = (double) studentMetrics.cyclomaticComplexity/(double)solutionMetrics.cyclomaticComplexity;
		cyclomaticComplexity = (double) studentMetrics.cyclomaticComplexity;
	}
	
	private void calculateOperatorsUsedRatio(){
		double studentOperatorsUsed = getBooleanOperatorsUsed(studentMetrics);
		double solutionOperatorsUsed = getBooleanOperatorsUsed(solutionMetrics);
		operatorsUsedRatio = studentOperatorsUsed/solutionOperatorsUsed;
	};
	
	private int getBooleanOperatorsUsed(ConditionalStatementsSummary metrics) {
		int operatorsUsed = 0;
		operatorsUsed += metrics.numberOfOperatorAnd>0?1:0;
		operatorsUsed += metrics.numberOfOperatorOr>0?1:0;
		operatorsUsed += metrics.numberOfOperatorEqual>0?1:0;
		operatorsUsed += metrics.numberOfOperatorNotEqual>0?1:0;
		operatorsUsed += metrics.numberOfOperatorLessThan>0?1:0;
		operatorsUsed += metrics.numberOfOperatorMoreThan>0?1:0;
		operatorsUsed += metrics.numberOfOperatorLessOrEqual>0?1:0;
		operatorsUsed += metrics.numberOfOperatorMoreOrEqual>0?1:0;
		operatorsUsed += metrics.numberOfBooleanFunction>0?1:0;
		return operatorsUsed; 
	}
	
	
	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		return "\nConditionalStatementBadge \n\n"
				+"\tNumber of If's ratio: " +df2.format(numberOfIfsRatio) +"\n"
				+"\tNumber of operators used ratio; "+ df2.format(operatorsUsedRatio) +"\n"
				+"\tStudent Cyclomatic complexity: "+ df2.format(cyclomaticComplexity) +"\n"
				+"\tCyclomatic complexity ratio; "+ df2.format(cyclomaticComplexityRatio)+"\n\n"
				+"\tThe total score is: " + df2.format(totalRatio/numberOfValidRatios) + "\n"
				+"\tThe Valor is: "+valor+"\n"
				+"\t\tBronze score limit: "+conditionalStatementMedalValues.get(2)+"\n"
				+"\t\tSilver score limit: "+conditionalStatementMedalValues.get(1)+"\n"
				+"\t\tGold score limit: "+conditionalStatementMedalValues.get(0)+"\n\n";
	}
	
	
}
