package badges;

import java.text.DecimalFormat;
import java.util.List;

import summary.metric.extraction.ExceptionMetricsSummary;
import summary.metric.extraction.MetricsSummary;

public class ExceptionMetricsBadge {
	private ExceptionMetricsSummary studentMetrics;
	private ExceptionMetricsSummary solutionMetrics;
	private List<Double> exceptionMetricMedalValues;
	private double totalRatio = 0;
	private double numberOfValidRatios = 0;
	public Valor valor;
	public double numberOfTryRatio;
	public double numberOfCatchRatio;
	public double numberOfThrowRatio;
	public double numberOfIllegalArgumentExceptions;
	
	public ExceptionMetricsBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> exceptionMetricMedalValues) {
		this.studentMetrics = studentMetricsSummary.exceptionMetricsSummary;
		this.solutionMetrics = solutionMetricsSummary.exceptionMetricsSummary;
		this.exceptionMetricMedalValues = exceptionMetricMedalValues;
		compareStudentMetrics();
		getValor();
	}
	
	
	private void compareStudentMetrics() {
		calculateNumberOfCatchRatio();
		calculateNumberOfThrowRatio();
		calculateNumberOfTryRatio();
		calculateNumberOfIllegalArgumentExceptions();
	}
	
	private void calculateNumberOfIllegalArgumentExceptions() {
		numberOfIllegalArgumentExceptions = (double) studentMetrics.numberOfIllegalArgumentExceptions/(double)solutionMetrics.numberOfIllegalArgumentExceptions;
	}

	private void calculateNumberOfTryRatio() {
		numberOfTryRatio = (double) studentMetrics.numberOfTryStatements/(double)solutionMetrics.numberOfTryStatements;
	}

	private void calculateNumberOfThrowRatio() {
		numberOfThrowRatio = (double) studentMetrics.numberOfThrowStatements/(double)solutionMetrics.numberOfThrowStatements;
	}

	public void calculateNumberOfCatchRatio() {
		numberOfCatchRatio = (double)studentMetrics.numberOfCatchStatements/(double)solutionMetrics.numberOfCatchStatements;
	}
	

	

	private void getValor() {


		if(!Double.isNaN(numberOfCatchRatio)) {
			totalRatio +=numberOfCatchRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(numberOfThrowRatio)) {
			totalRatio +=numberOfThrowRatio;
			numberOfValidRatios ++;
		}		
		if(!Double.isNaN(numberOfTryRatio)) {
			totalRatio +=numberOfTryRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(numberOfIllegalArgumentExceptions)) {
			totalRatio +=numberOfIllegalArgumentExceptions;
			numberOfValidRatios ++;
		}
		if (totalRatio == 0 ) {
			valor=Valor.NONE;
		}
		else if (totalRatio/numberOfValidRatios >= exceptionMetricMedalValues.get(1))
			valor=Valor.GOLD;
		else if(totalRatio/numberOfValidRatios >= exceptionMetricMedalValues.get(2))
			valor=Valor.SILVER;
		else if(totalRatio/numberOfValidRatios >= exceptionMetricMedalValues.get(3))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}
	
	}
	
	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		return "\nExceptionMetricsBadge \n\n"
				+"\tNumber of try ratio: " +df2.format(numberOfTryRatio) +"\n"
				+"\tNumber of catch ratio: "+ df2.format(numberOfCatchRatio) +"\n"
				+"\tNumber of throw ratio: "+ df2.format(numberOfThrowRatio)+"\n\n"
				+"\tNumber of illegalArgumentException ratio: "+ df2.format(numberOfIllegalArgumentExceptions)+"\n\n"
				+"\tThe total score is: " + df2.format(totalRatio/numberOfValidRatios) + "\n"
				+"\tThe Valor is: "+valor +"\n"
				+"\t\tBronze score limit: "+exceptionMetricMedalValues.get(2)+"\n"
				+"\t\tSilver score limit: "+exceptionMetricMedalValues.get(1)+"\n"
				+"\t\tGold score limit: "+exceptionMetricMedalValues.get(0)+"\n\n";
	}
	
	
}
