package badges;

import java.text.DecimalFormat;
import java.util.List;

import summary.metric.extraction.GeneralMetricsSummary;
import summary.metric.extraction.MetricsSummary;

public class GeneralMetricsBadge {
	private GeneralMetricsSummary studentMetrics;
	private GeneralMetricsSummary solutionMetrics;
	private List<Double> generalMetricMedalValues;
	private double numberOfValidRatios = 0;
	private double totalRatio = 0;
	public Valor valor;
	public double linesTotalRatio;
	public double methodCallsTotalRatio;
	public double variableDeclarationRatio;
	public double weightedMethodPerClass;
	
	public GeneralMetricsBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> generalMetricMedalValues) {
		this.studentMetrics = studentMetricsSummary.generalMetricsSummary;
		this.solutionMetrics = solutionMetricsSummary.generalMetricsSummary;
		this.generalMetricMedalValues = generalMetricMedalValues;
		compareStudentMetrics();
		getValor();
	}
	
	private void createWeightedMethodsPerClass() {
		double complexityRatio = (double)studentMetrics.hightestMethodComplexity/(double)solutionMetrics.hightestMethodComplexity;
		double numberOfMethodsRatio = (double)studentMetrics.totalNumberOfMethods/(double)solutionMetrics.totalNumberOfMethods;
		double sizeOfMethodsRatio = (double)studentMetrics.largestMethodSize/(double)solutionMetrics.largestMethodSize;
		weightedMethodPerClass =(complexityRatio+numberOfMethodsRatio)/2;
		numberOfMethodsRatio ++;
	
	}

	private void getValor() {

		if(!Double.isNaN(linesTotalRatio) && !Double.isInfinite(linesTotalRatio)) {
			totalRatio +=linesTotalRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(methodCallsTotalRatio) && !Double.isInfinite(methodCallsTotalRatio)) {
			totalRatio +=methodCallsTotalRatio;
			numberOfValidRatios ++;
		}		
		if(!Double.isNaN(variableDeclarationRatio) && !Double.isInfinite(variableDeclarationRatio)) {
			totalRatio +=variableDeclarationRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(weightedMethodPerClass) && !Double.isInfinite(weightedMethodPerClass)) {
			System.out.println("Weighted method per class " + weightedMethodPerClass);
			totalRatio +=weightedMethodPerClass;
			numberOfValidRatios ++;
		}

		if (totalRatio/numberOfValidRatios <= generalMetricMedalValues.get(0))
			valor=Valor.GOLD;
		else if(totalRatio/numberOfValidRatios <= generalMetricMedalValues.get(1))
			valor=Valor.SILVER;
		else if(totalRatio/numberOfValidRatios <= generalMetricMedalValues.get(2))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}
	
	}

	private void compareStudentMetrics() {
		createWeightedMethodsPerClass();
		createLinesTotalRatio();
		createMethodCallsTotalRatio();
		createVariableDeclarationRatio();
		
	}
	
	private void createVariableDeclarationRatio() {
		variableDeclarationRatio = (double) studentMetrics.variableDeclarationsTotal/(double)solutionMetrics.variableDeclarationsTotal;
	}

	private void createMethodCallsTotalRatio() {
		methodCallsTotalRatio = (double) studentMetrics.methodCallsTotal/(double)solutionMetrics.methodCallsTotal;		
	}

	private void createLinesTotalRatio() {
		linesTotalRatio = (double)studentMetrics.linesTotal/(double)solutionMetrics.linesTotal;
	}

	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		return "\nGeneralMetricsBadge \n\n"
				+"\tNumber of lines ratio: " +df2.format(linesTotalRatio) +"\n"
				+"\tNumber of methods used ratio: "+ df2.format(methodCallsTotalRatio) +"\n"
				+"\tNumber of variableDeclarations ratio: "+ df2.format(variableDeclarationRatio)+"\n\n"
				+"\tWeighted methods per class Ratio: "+ df2.format(weightedMethodPerClass)+"\n\n"
				+"\tThe total score is: " + df2.format(totalRatio/numberOfValidRatios) + "\n"					
				+"\tThe Valor is: "+valor +"\n" 
				+"\t\tBronze score limit: "+generalMetricMedalValues.get(2)+"\n"
				+"\t\tSilver score limit: "+generalMetricMedalValues.get(1)+"\n"
				+"\t\tGold score limit: "+generalMetricMedalValues.get(0)+"\n\n";
	}
	
	
}
