package badges;

import java.text.DecimalFormat;
import java.util.List;

import summary.metric.extraction.InheritanceMetricsSummary;
import summary.metric.extraction.MetricsSummary;

public class InheritanceMetricsBadge {
	private InheritanceMetricsSummary studentInheritanceMetrics;
	private InheritanceMetricsSummary solutionInheritanceMetrics;
	private List<Double> inheritanceMetricMedalValues;
	private double numberOfValidRatios = 0;
	private double totalRatio = 0;
	public Valor valor;
	public double classOrInterfaceInheritanceRatio;
	public double superExpressionsRatio;
	
	public InheritanceMetricsBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> inheritanceMetricMedalValues) {
		this.studentInheritanceMetrics = studentMetricsSummary.inheritanceMetricsSummary;
		this.solutionInheritanceMetrics = solutionMetricsSummary.inheritanceMetricsSummary;
		this.inheritanceMetricMedalValues = inheritanceMetricMedalValues;
		compareStudentMetrics();
		getValor();
	}

	private void getValor() {

		
		if(!Double.isNaN(classOrInterfaceInheritanceRatio)) {
			totalRatio +=classOrInterfaceInheritanceRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(superExpressionsRatio)) {
			totalRatio +=superExpressionsRatio;
			numberOfValidRatios ++;
		}

		if (totalRatio/numberOfValidRatios <= inheritanceMetricMedalValues.get(0))
			valor=Valor.GOLD;
		else if(totalRatio/numberOfValidRatios <= inheritanceMetricMedalValues.get(1))
			valor=Valor.SILVER;
		else if(totalRatio/numberOfValidRatios <= inheritanceMetricMedalValues.get(2))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}
		
	}

	private void compareStudentMetrics() {
			createInheritanceRatio();
			createSuperExprUsageRatio();
	}
	
	private void createInheritanceRatio () {
		if (solutionInheritanceMetrics.totalAbstractClassOrInterfaceInheritance >0)
		classOrInterfaceInheritanceRatio = studentInheritanceMetrics.totalAbstractClassOrInterfaceInheritance/solutionInheritanceMetrics.totalAbstractClassOrInterfaceInheritance;
	}

	private void createSuperExprUsageRatio(){
		if (solutionInheritanceMetrics.totalSuperExprList >0)
			superExpressionsRatio = studentInheritanceMetrics.totalSuperExprList/solutionInheritanceMetrics.totalSuperExprList;
	} 
	
	
	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		if (valor != valor.NONE) {
			return "\nInheritanceMetricsBadge \n\n"
					+"\tNumber of interface or abstract classes implemented ratio: " +df2.format(classOrInterfaceInheritanceRatio) +"\n"
					+"\tNumber of super expressions used ratio: " +df2.format(superExpressionsRatio) +"\n\n"						
					+"\tThe total score is: " + df2.format(totalRatio/numberOfValidRatios) + "\n"
					+"\tThe Valor is: "+valor +"\n" 
					+"\t\tBronze score limit: "+inheritanceMetricMedalValues.get(2)+"\n"
					+"\t\tSilver score limit: "+inheritanceMetricMedalValues.get(1)+"\n"
					+"\t\tGold score limit: "+inheritanceMetricMedalValues.get(0)+"\n\n";
		}
		else {
			return "";
		}
	}
	
	
}
