package badges;

import java.text.DecimalFormat;
import java.util.List;

import summary.metric.extraction.IterableMetricsSummary;
import summary.metric.extraction.IteratorMetricsSummary;
import summary.metric.extraction.MetricsSummary;

public class IterationMetricsBadge {
	private MetricsSummary studentMetrics;
	private MetricsSummary solutionMetrics;
	private List<Double> iterationMetricMedalValues;
	private double numberOfValidRatios = 0;
	private double totalRatio = 0;
	public Valor valor;
	public double iteratorsUsedRatio;
	public double iterablesUsedRatio;
	
	public IterationMetricsBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> iterationMetricMedalValues) {
		this.studentMetrics = studentMetricsSummary;
		this.solutionMetrics = solutionMetricsSummary;
		this.iterationMetricMedalValues = iterationMetricMedalValues;
		compareStudentMetrics();
		getValor();
	}

	private void getValor() {

		
		if(!Double.isNaN(iterablesUsedRatio)) {
			totalRatio +=iterablesUsedRatio;
			numberOfValidRatios ++;
		}
		if(!Double.isNaN(iteratorsUsedRatio)) {
			totalRatio +=iteratorsUsedRatio;
			numberOfValidRatios ++;
		}

		if (totalRatio/numberOfValidRatios <= iterationMetricMedalValues.get(0))
			valor=Valor.GOLD;
		else if(totalRatio/numberOfValidRatios <= iterationMetricMedalValues.get(1))
			valor=Valor.SILVER;
		else if(totalRatio/numberOfValidRatios <= iterationMetricMedalValues.get(2))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}
		
	}

	private void compareStudentMetrics() {
			createIteratorsUsedMetrics();
			createIterablesUsedMetrics();
	}
	
	private void createIteratorsUsedMetrics(){
		double studentOperatorsUsed = getIteratorsUsed(studentMetrics.iteratorMetricsSummary);
		double solutionOperatorsUsed = getIteratorsUsed(solutionMetrics.iteratorMetricsSummary);
		iteratorsUsedRatio =  studentOperatorsUsed/solutionOperatorsUsed;
	};
	
	private int getIterablesUsed(IterableMetricsSummary metrics) {
		int operatorsUsed = 0;
		operatorsUsed += metrics.totalArrayAccesses>0?1:0;
		operatorsUsed += metrics.totalArrayCreations>0?1:0;
		operatorsUsed += metrics.totalArrayInit>0?1:0;
		return operatorsUsed; 
	}
	
	private void createIterablesUsedMetrics(){
		double studentIterablesUsed = getIterablesUsed(studentMetrics.iterableMetricsSummary);
		double solutionIterablesUsed = getIterablesUsed(solutionMetrics.iterableMetricsSummary);
		iterablesUsedRatio = studentIterablesUsed/solutionIterablesUsed;
	};
	
	private int getIteratorsUsed(IteratorMetricsSummary metrics) {
		int operatorsUsed = 0;
		operatorsUsed += metrics.totalWhileLoops>0?1:0;
		operatorsUsed += metrics.totalForLoops>0?1:0;
		operatorsUsed += metrics.totalLambdaExpr>0?1:0;
		operatorsUsed += metrics.totalForeachLoops>0?1:0;
		
		return operatorsUsed; 
	}
	
	
	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		if (valor != valor.NONE) {
			return "\nIterationMetricsBadge \n\n"
					+"\tNumber of iterators used ratio: " +df2.format(iteratorsUsedRatio) +"\n"
					+"\tNumber of iterables used ratio: " +df2.format(iterablesUsedRatio) +"\n\n"						
					+"\tThe total score is: " + df2.format(totalRatio/numberOfValidRatios) + "\n"
					+"\tThe Valor is: "+valor +"\n" 
					+"\t\tBronze score limit: "+iterationMetricMedalValues.get(2)+"\n"
					+"\t\tSilver score limit: "+iterationMetricMedalValues.get(1)+"\n"
					+"\t\tGold score limit: "+iterationMetricMedalValues.get(0)+"\n\n";
		}
		else {
			return "";
		}
	}
	
	
}
