package badges;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import list.printer.ListPrinter;
import summary.metric.extraction.ConditionalStatementsSummary;
import summary.metric.extraction.ExceptionMetricsSummary;
import summary.metric.extraction.GeneralMetricsSummary;
import summary.metric.extraction.IterableMetricsSummary;
import summary.metric.extraction.IteratorMetricsSummary;
import summary.metric.extraction.MetricsSummary;

public class OverallMetricsBadge {
	private IteratorMetricsSummary studentIteratorMetrics;
	private IterableMetricsSummary studentIterableMetrics;
	private ConditionalStatementsSummary studentConditionalStatementMetrics;
	private GeneralMetricsSummary studentGeneralMetrics;
	private ExceptionMetricsSummary studentExceptionMetrics;
	private List<Double> overallMetricMedalValues;
	private double metricsUsedRatio;
	
	public List<String> unusedMetrics = new ArrayList<>();
	public List<String> usedMetrics = new ArrayList<>();
	public Valor valor;

	
	public OverallMetricsBadge(MetricsSummary studentMetricsSummary, MetricsSummary solutionMetricsSummary, List<Double> overallMetricMedalValues) {
		this.studentIteratorMetrics = studentMetricsSummary.iteratorMetricsSummary;
		this.studentIterableMetrics = studentMetricsSummary.iterableMetricsSummary;
		this.studentConditionalStatementMetrics = studentMetricsSummary.conditionalStatementsMetrics;
		this.studentGeneralMetrics = studentMetricsSummary.generalMetricsSummary;
		this.studentExceptionMetrics = studentMetricsSummary.exceptionMetricsSummary;
		this.overallMetricMedalValues = overallMetricMedalValues;
		
		compareStudentMetrics();
		getValor();
	}
	
	public OverallMetricsBadge(MetricsSummary metricsSummary) {
		this.studentIteratorMetrics = metricsSummary.iteratorMetricsSummary;
		this.studentIterableMetrics = metricsSummary.iterableMetricsSummary;
		this.studentConditionalStatementMetrics = metricsSummary.conditionalStatementsMetrics;
		this.studentGeneralMetrics = metricsSummary.generalMetricsSummary;
		this.studentExceptionMetrics = metricsSummary.exceptionMetricsSummary;
		
		compareStudentMetrics();
	}

	private void getValor() {
		double totalNumberOfMetrics = (double) unusedMetrics.size()+ (double)usedMetrics.size();
		metricsUsedRatio = usedMetrics.size()/totalNumberOfMetrics;
	
		if (metricsUsedRatio >= overallMetricMedalValues.get(0))
			valor=Valor.GOLD;
		else if(metricsUsedRatio >= overallMetricMedalValues.get(1))
			valor=Valor.SILVER;
		else if(metricsUsedRatio >= overallMetricMedalValues.get(2))
			valor=Valor.BRONZE;
		else {
			valor=Valor.NONE;
		}
	}
	
	private void addToListIfTrue(String name,Boolean exists) {
		if(exists) {
			usedMetrics.add(name);
		}
		
		else {
			unusedMetrics.add(name);
		}
	}

	private void compareStudentMetrics() {
		//Iterator	
		addToListIfTrue("ForEach", studentIteratorMetrics.totalForeachLoops>0);
		addToListIfTrue("For",studentIteratorMetrics.totalForLoops>0);
		addToListIfTrue("While", studentIteratorMetrics.totalWhileLoops>0);
		addToListIfTrue("Lambda", studentIteratorMetrics.totalLambdaExpr>0);
	
		
		//Iterable
		addToListIfTrue("ArrayInit", studentIterableMetrics.totalArrayInit>0);
		addToListIfTrue("ArrayCreation", studentIterableMetrics.totalArrayCreations>0);
		addToListIfTrue("ArrayAccess", studentIterableMetrics.totalArrayAccesses>0);
			
		
		//ConditionalStatements
		addToListIfTrue("Else", studentConditionalStatementMetrics.numberOfElses>0);
		addToListIfTrue("Ifs", studentConditionalStatementMetrics.numberOfIfs>0);
		addToListIfTrue("Switches", studentConditionalStatementMetrics.numberOfSwitches>0);
		addToListIfTrue("BooleanOperatorFunction", studentConditionalStatementMetrics.numberOfBooleanFunction>0);
		addToListIfTrue("BooleanOperatorAnd", studentConditionalStatementMetrics.numberOfOperatorAnd>0);
		addToListIfTrue("BooleanOperatorOr", studentConditionalStatementMetrics.numberOfOperatorOr>0);
		addToListIfTrue("BooleanOperatorLessThan", studentConditionalStatementMetrics.numberOfOperatorLessThan>0);
		addToListIfTrue("BooleanOperatorMoreThan", studentConditionalStatementMetrics.numberOfOperatorMoreThan>0);
		addToListIfTrue("BooleanOperatorNotEqual", studentConditionalStatementMetrics.numberOfOperatorNotEqual>0);
		addToListIfTrue("BooleanOperatorEqual", studentConditionalStatementMetrics.numberOfOperatorEqual>0);
		addToListIfTrue("BooleanOperatorMoreOrEqual", studentConditionalStatementMetrics.numberOfOperatorMoreOrEqual>0);
		addToListIfTrue("BooleanOperatorLessOrEqual", studentConditionalStatementMetrics.numberOfOperatorLessOrEqual>0);
			
		//General
		addToListIfTrue("UnaryOperatorAddition", studentGeneralMetrics.numberOfAddition>0);
		addToListIfTrue("UnaryOperatorSubtraction", studentGeneralMetrics.numberOfSubtraction>0);
		addToListIfTrue("UnaryOperatorPlusOne", studentGeneralMetrics.numberOfPlusOne>0);
		addToListIfTrue("UnaryOperatorMinusOne", studentGeneralMetrics.numberOfMinusOne>0);
		addToListIfTrue("UnaryOperatorMultiply", studentGeneralMetrics.numberOfMultiply>0);
		addToListIfTrue("UnaryOperatorDivide", studentGeneralMetrics.numberOfDivide>0);
		addToListIfTrue("UnaryOperatorRemains", studentGeneralMetrics.numberOfRemains>0);
		addToListIfTrue("UnaryOperatorBooleanInverter", studentGeneralMetrics.numberOfBooleanInverter>0);
		addToListIfTrue("PrivateFieldDeclaration", studentGeneralMetrics.numberOfPrivate>0);
		addToListIfTrue("PublicFieldDeclaration", studentGeneralMetrics.numberOfPublic>0);
		addToListIfTrue("ProtectedFieldDeclaration", studentGeneralMetrics.numberOfProtected>0);
		
		
		//Exception
		addToListIfTrue("CatchStatements", studentExceptionMetrics.numberOfCatchStatements>0);
		addToListIfTrue("ThrowStatements", studentExceptionMetrics.numberOfThrowStatements>0);
		addToListIfTrue("TryStatements", studentExceptionMetrics.numberOfTryStatements>0);
		addToListIfTrue("IllegalArgumentExcpetions", studentExceptionMetrics.numberOfIllegalArgumentExceptions>0);
			//iteratorsUsedRatio = iteratorsUsedMetrics();
	}
	
	private static DecimalFormat df2 = new DecimalFormat(".##");
	
	public String toString() {
		return  "\nOverallMetricsBadge\n\t"+ new ListPrinter(unusedMetrics, "\tUnused Metrics: ") + 
        "\n\tMetrics used/Metrics total: " + +usedMetrics.size()+ " / " +(this.usedMetrics.size()+this.unusedMetrics.size())+ 
        "\n\tThe total score is: " + df2.format(metricsUsedRatio)+ 
        "\n\tValor: " + this.valor +"\n"
		+"\t\tBronze score limit: "+overallMetricMedalValues.get(2)+"\n"
		+"\t\tSilver score limit: "+overallMetricMedalValues.get(1)+"\n"
		+"\t\tGold score limit: "+overallMetricMedalValues.get(0)+"\n\n";
	}
}
