package list.printer;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import badges.ExceptionMetricsBadge;
import badges.OverallMetricsBadge;
import file.handling.SolutionPathGenerator;
import metrics.collection.MetricsCollectionList;
import summary.metric.extraction.MetricsSummary;

public class SolutionBadgeSimulation {
	private String simulationBasePath;
	private ArrayList<String> listOfTasks;
	public OverallMetricsBadge overallMetricsBadge;
	public ExceptionMetricsBadge exceptionMetricsBadge;
	
	public SolutionBadgeSimulation(String simulationBasePath, ArrayList<String> listOfTasks) {
		this.simulationBasePath = simulationBasePath;
		this.listOfTasks = listOfTasks;
		simulateIndividualTaskUsage();
	}
	//Simulation of badges 

	public String printUnusedMetrics() {
		return "\n There are "+ overallMetricsBadge.unusedMetrics.size()+" number of unused metrics \n" +overallMetricsBadge.unusedMetrics + "";
	}
	
	
	public ArrayList<String> relevantMetricsList(MetricsSummary solutionMetricsSummary) {
		ArrayList<String> relevantMetrics = new ArrayList<String>();
		
		int sumOfExceptionMetricUsage = solutionMetricsSummary.exceptionMetricsSummary.numberOfCatchStatements + solutionMetricsSummary.exceptionMetricsSummary.numberOfTryStatements + solutionMetricsSummary.exceptionMetricsSummary.numberOfIllegalArgumentExceptions + solutionMetricsSummary.exceptionMetricsSummary.numberOfThrowStatements; 
		if(sumOfExceptionMetricUsage>3){relevantMetrics.add("ExceptionMetrics");}	
		
		int sumOfInheritanceMetricsUsage = solutionMetricsSummary.inheritanceMetricsSummary.totalAbstractClassOrInterfaceInheritance + solutionMetricsSummary.inheritanceMetricsSummary.totalSuperExprList;
		if(sumOfInheritanceMetricsUsage>3) {relevantMetrics.add("InheritanceMetrics");}
		
		int sumOfConditionalStatementUsage = solutionMetricsSummary.conditionalStatementsMetrics.numberOfElses + solutionMetricsSummary.conditionalStatementsMetrics.numberOfIfs + solutionMetricsSummary.conditionalStatementsMetrics.numberOfSwitches;
		if(sumOfConditionalStatementUsage>3) {relevantMetrics.add("ConditionalStatementMetrics");}
		
		int sumOfIterationMetricsUsage = solutionMetricsSummary.iterableMetricsSummary.totalArrayAccesses + solutionMetricsSummary.iterableMetricsSummary.totalArrayCreations + solutionMetricsSummary.iterableMetricsSummary.totalArrayInit + solutionMetricsSummary.iteratorMetricsSummary.totalForeachLoops + solutionMetricsSummary.iteratorMetricsSummary.totalForLoops + solutionMetricsSummary.iteratorMetricsSummary.totalWhileLoops + solutionMetricsSummary.iteratorMetricsSummary.totalLambdaExpr; 
		if(sumOfIterationMetricsUsage>3) {relevantMetrics.add("IterationMetrics");}
		
		return relevantMetrics;
	}
	
	
	private void simulateIndividualTaskUsage() {
		for (String task:listOfTasks){
			File solutionProjectDir = new File(simulationBasePath+task);
			SolutionPathGenerator solutionPathGenerator = new SolutionPathGenerator(solutionProjectDir); 
			MetricsCollectionList solutionMetricsCollectionList = new MetricsCollectionList(solutionPathGenerator.pathList, simulationBasePath+task);
			MetricsSummary solutionMetricsSummary = new MetricsSummary(solutionMetricsCollectionList);
			overallMetricsBadge = new OverallMetricsBadge(solutionMetricsSummary);
			
			//System.out.println("\nTask name: " +task+"\n" + solutionMetricsSummary.printCyclomaticComplexity());			
			//System.out.println("task " +task + printUnusedMetrics());
			
			ArrayList<String> listOfRelevantMetrics = relevantMetricsList(solutionMetricsSummary);
			System.out.println(task + "\n");
			for(String metric:listOfRelevantMetrics) {
				System.out.println("\t"+metric + "\n");
			}
			
			
		}
	}
	
}
