package metrics.collection;

import com.github.javaparser.ast.CompilationUnit;

import metrics.conditional.statements.ConditionalStatementsMetric;
import metrics.exceptions.ExceptionMetrics;
import metrics.general.GeneralMetrics;
import metrics.inheritance.InheritanceMetrics;
import metrics.iterable.IterableMetrics;
import metrics.iterator.IteratorMetrics;
import metrics.java.standard.methods.JavaStandardClassMethodsMetric;



public class MetricsCollection {
	public CompilationUnit compilationUnit;
	public IteratorMetrics iteratorMetrics;
	public GeneralMetrics generalMetrics;
	public ConditionalStatementsMetric conditionalStatementsMetrics;
	public ExceptionMetrics exceptionMetrics;
	public IterableMetrics iterableMetrics;
	public InheritanceMetrics inheritanceMetrics;
	public JavaStandardClassMethodsMetric javaStandardClassMethodsMetrics;
	
	
	public MetricsCollection(CompilationUnit compilationUnit) {
		this.compilationUnit=compilationUnit;
		calculateAllMetrics();
	}
	
	public void calculateAllMetrics() {
		//Calculating the iterator metric. 
	    this.iteratorMetrics = new IteratorMetrics(compilationUnit);
    
	    //Finding the general metric. 
	    this.generalMetrics = new GeneralMetrics(compilationUnit);
	    
	    //Finding the conditional statements metric. 
	    this.conditionalStatementsMetrics = new ConditionalStatementsMetric(compilationUnit);
	    
	    //Finding exceptionMetrics; 
	    this.exceptionMetrics = new ExceptionMetrics(compilationUnit);
	    
	    //Finding iteration metrics
	    this.iterableMetrics = new IterableMetrics(compilationUnit);

	    //Finding inheritance metrics 
	    this.inheritanceMetrics = new InheritanceMetrics(compilationUnit);

	    //Finding java standard class methods metrics. 
	    this.javaStandardClassMethodsMetrics = new JavaStandardClassMethodsMetric(compilationUnit);

	}
	
	public String toString() {
		return  iteratorMetrics + "" +
				generalMetrics + 
				conditionalStatementsMetrics + 
				exceptionMetrics + 
				iterableMetrics + 
				inheritanceMetrics + 
				javaStandardClassMethodsMetrics
				; 
	}
    
}
