package metrics.conditional.statements;

import java.util.ArrayList;
import java.util.List;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.expr.Expression;
import com.github.javaparser.ast.stmt.IfStmt;
import com.github.javaparser.ast.stmt.SwitchStmt;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;
import list.printer.ListPrinter;
import metrics.general.CyclomaticComplexityGenerator;

import com.github.javaparser.ast.stmt.Statement;


//TODO: Add elseStatement list. 
//TODO: Find more information about the expressions in the statements.(&&, ||, !=, ==, .equals, etc..)
public class ConditionalStatementsMetric {
	private CompilationUnit source;
	public List<IfStmt> ifStmtList = new ArrayList<>();
	public List<Expression> conditionList = new ArrayList<>();
	public List<SwitchStmt> switchStmtList = new ArrayList<>();
	public ConditionalOperatorCounter conditionalOperatorCounter;
	public int cyclomaticComplexity;
	//private TypeSolver typeSolver;
	
	public ConditionalStatementsMetric(CompilationUnit source) {
		this.source=source;
		calculateMetrics();
		//this.typeSolver=typeSolver;
	}
	
	public void calculateMetrics() {
		createIfStmtList();
		createSwitchStmtList();
		createCyclomaticComplexity();
	}
	

	private void createIfStmtList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final IfStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	ifStmtList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
		
		createConditionList();
		
	}
	
	private void createConditionList() { 
		
		for(IfStmt ifStatement:ifStmtList){
			conditionList.add(ifStatement.getCondition());				
		}
		createOperatorList();
	}
	
	private void createOperatorList() {
		conditionalOperatorCounter = new ConditionalOperatorCounter(conditionList);
	}
	
	private void createCyclomaticComplexity() {
		CyclomaticComplexityGenerator complexityGenerator = new CyclomaticComplexityGenerator(source);
	    cyclomaticComplexity = complexityGenerator.getCyclomaticComplexity();
	}

	
	
	private void createSwitchStmtList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final SwitchStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	switchStmtList.add(n);			    
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	//TODO add Ternary operators as well. 
	
    public String toString() {
    	if (ifStmtList.isEmpty() && switchStmtList.isEmpty())
    		return "";
    	else { 	
    		return "CONDITIONAL STATEMENT METRICS: \n\n" + 
            		new ListPrinter(ifStmtList, "If Statements: ")
            		+ new ListPrinter(switchStmtList, "Switch Statements: ")
            		+ conditionalOperatorCounter;   	
    		}
        
    }
	
}