package metrics.exceptions;

import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.StringUtils;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.stmt.CatchClause;
import com.github.javaparser.ast.stmt.ThrowStmt;
import com.github.javaparser.ast.stmt.TryStmt;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;
import list.printer.ListPrinter;

public class ExceptionMetrics {
	
	private CompilationUnit source; 
	public List<TryStmt> tryStmtList = new ArrayList<>();
	public List<ThrowStmt> throwStmtList = new ArrayList<>();
	public List<CatchClause> catchStmtList = new ArrayList<>();
	public int numberOfArgumentExceptions = 0;
	

	public ExceptionMetrics(CompilationUnit source) {
		//check if null and throw exception. 
		this.source = source;
		calculateMetrics();

	}
	
	
	public void calculateMetrics() {
		createTryStatementList();
		createThrowStatementList();
		createCatchStatementList();
	}
	
	public void countExceptions() {
		//ThrowIllegalArgument
		for (ThrowStmt throwStmt:throwStmtList) {
			numberOfArgumentExceptions += StringUtils.countMatches(throwStmt.toString(), "IllegalArgumentException");
		}
	}
			
	private void createTryStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final TryStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	tryStmtList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}

	private void createThrowStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ThrowStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	throwStmtList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
		countExceptions();
	}
	
	private void createCatchStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final CatchClause n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	catchStmtList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
    public String toString() {
    	if (tryStmtList.isEmpty() && catchStmtList.isEmpty() && throwStmtList.isEmpty())
    		return "";
    	else { 	
    		return "EXCEPTIONS METRICS: \n\n" + new ListPrinter(tryStmtList, "Try Statements : ")
        		+ new ListPrinter(catchStmtList, "Catch Statements: ")
        		+ new ListPrinter(throwStmtList, "Throw Statements : ");
    	}
    }
	
}
