package metrics.general;


import org.apache.commons.lang3.StringUtils;

import com.github.javaparser.ast.Node;
import com.github.javaparser.ast.comments.Comment;
import com.github.javaparser.ast.stmt.IfStmt;
import com.github.javaparser.ast.stmt.Statement;

public class CyclomaticComplexityGenerator {
	private Node node;
    private int complexity = 0;
    
    
    public CyclomaticComplexityGenerator(Node node) {
    	this.node = node;
    }
    
    static void removeComments(Node node) {
        for (Comment child : node.getAllContainedComments()) {
            child.remove();
        }
    }
    
    
    public int getCyclomaticComplexity() {
    	for (IfStmt ifStmt : node.getChildNodesByType(IfStmt.class)) {
            // adds one after finding if 
            complexity++;
            if (ifStmt.getElseStmt().isPresent()) {
                // check if, if has else
            	Statement elseStmt = ifStmt.getElseStmt().get();
                if (elseStmt instanceof IfStmt) {
                    // it's an "else-if". We already count that by counting the "if" above.
                } else {
                    // it's an "else-something". Add it.
                    complexity++;
                }
            }
        }
    	
        
    	//Checking for other the "?" if, and "||" (or) that also counts as cyclomaticComplexity
    	
    	for (Node node : node.getChildNodes()) {
    		removeComments(node);
    		complexity += StringUtils.countMatches(node.toString(), "?");
			complexity += StringUtils.countMatches(node.toString(), "||");
    	}
    	
    	
       	return complexity;
    	
    }
    
    
}
