package metrics.general;

import java.util.List;
import org.apache.commons.lang3.StringUtils;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.Node;
import com.github.javaparser.ast.body.BodyDeclaration;
import com.github.javaparser.ast.body.FieldDeclaration;

public class FieldDeclarations {
	private CompilationUnit source;
	public int numberOfPrivate;
	public int numberOfPublic;
	public int numberOfProtected;
	
	public FieldDeclarations(CompilationUnit source) {
		this.source = source;
		createFieldDeclarationList();
	}
	
	private void createFieldDeclarationList() {
		BodyDeclaration body = getBody(source);
		if (body != null) {	
			List<Node> nodes = body.getChildNodes();
			for (Node node : nodes) {

				if (node instanceof FieldDeclaration) {
					numberOfPrivate += StringUtils.countMatches(node.toString(), "private");
					numberOfProtected += StringUtils.countMatches(node.toString(), "protected");
					numberOfPublic += StringUtils.countMatches(node.toString(), "public");
				}				
			}
		}
	}

	private static BodyDeclaration getBody(CompilationUnit cu) {
		List<Node> nodes = cu.getChildNodes();
		for (Node node : nodes) {
			if (node instanceof BodyDeclaration) {
				return (BodyDeclaration) node;
				
			}
		}
		return null;
	}
	
	public String toString() {
		return "\tFieldDeclarations\n"+
				"\t\tNumber of private: " + numberOfPrivate + "\n"+
				"\t\tNumber of public: " + numberOfPublic + "\n"+
				"\t\tNumber of protected: " + numberOfProtected + "\n";
	}
	
}
