package metrics.general;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.body.MethodDeclaration;
import com.github.javaparser.ast.expr.MethodCallExpr;
import com.github.javaparser.ast.expr.UnaryExpr;
import com.github.javaparser.ast.expr.VariableDeclarationExpr;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;
import list.printer.ListPrinter;



public class GeneralMetrics {
	private List<String> unaryExprList = new ArrayList<>();
	private CompilationUnit source; 
	public int linesCount = 0;
	public List<MethodCallExpr> methodCallsList = new ArrayList<>();
	public List<VariableDeclarationExpr> variableDeclarationExprList = new ArrayList<>();
	public FieldDeclarations fieldDeclarations;
	public UnaryOperators unaryOperators;
	public GeneralSymbolSolver generalSymbolSolver;
	public int numberOfMethods;
	public int largestMethod;
	public int hightestMethodComplexity;

	public GeneralMetrics(CompilationUnit source) {
		//check if null and throw exception. 
		this.source = source;
		calculateMetrics();

	}
	
	
	public void calculateMetrics() {
		this.linesCount = countLines(this.source.toString());
		createFieldDeclarations();
		createMethodCallsList();
		createVariableDeclarationList();
		createUnaryExprList();
		
		//Theres a bug the two functions below, that occurs occationally.
		//Especcially with the simulation of the proposed solution 
		
		//createGeneralSymbolSolver();
		//createWheightedMethodsPerClass();
	}
	

	private int countLines(String source) {
		boolean newLine = true;
		int lineCount = 0;
		for (int i = 0; i < source.length(); i++) {
			char c = source.charAt(i);
			if (c == '\n') {
				newLine = true;
			} else if (newLine && (! Character.isWhitespace(c))) {
				lineCount++;
				newLine = false;
			}
		}
		return lineCount;
	}
	
	private void createGeneralSymbolSolver() {
		this.generalSymbolSolver = new GeneralSymbolSolver(source);
	}
	
	private void createFieldDeclarations() {
		this.fieldDeclarations = new FieldDeclarations(source);
	}
	
	private void createUnaryExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final UnaryExpr n, final Void arg){
			    	unaryExprList.add(n.toString());
	                super.visit(n, arg);
	            };
	            
	        }, null);
		createUnaryOperatorList();
	}
	
	private void createWheightedMethodsPerClass() {
		List<Integer> lengthOfMethodList = new ArrayList<>();
		List<Integer> complexityOfMethodList = new ArrayList<>();
		source.accept(new VoidVisitorAdapter<Void>(){
			@Override
            public void visit(final MethodDeclaration n, final Void arg){
              //  System.out.println("If statement :"+n);
		    	//variableDeclarationExprList.add(n);
		    	lengthOfMethodList.add(countLines(n.toString()));
		    	complexityOfMethodList.add(new CyclomaticComplexityGenerator(n).getCyclomaticComplexity());
                super.visit(n, arg);
            };
             
        }, null);
		numberOfMethods = lengthOfMethodList.size();
		largestMethod = Collections.max(lengthOfMethodList);
		hightestMethodComplexity = Collections.max(complexityOfMethodList);
	}
	
	private void createUnaryOperatorList() {
		this.unaryOperators = new UnaryOperators(unaryExprList);
	}
	
	private void createVariableDeclarationList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final VariableDeclarationExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	variableDeclarationExprList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
		
	private void createMethodCallsList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final MethodCallExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	methodCallsList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	

    public String toString() {
    	if (methodCallsList.isEmpty() && linesCount == 0 && variableDeclarationExprList.isEmpty())
    		return "";
    	else { 	
    		return "GENERAL METRICS: \n\n" + 
    				"Number of lines : " + this.linesCount + "\n\n" +
            		new ListPrinter(methodCallsList, "Methods used: ")
            		+ fieldDeclarations
            		+ new ListPrinter(variableDeclarationExprList, "Variable declaration expression list: ");   	
    		}
        }
}
