package metrics.inheritance;

import java.util.ArrayList;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
import com.github.javaparser.ast.expr.SuperExpr;
import com.github.javaparser.ast.type.ClassOrInterfaceType;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;

import list.printer.ListPrinter;

public class InheritanceMetrics {
	CompilationUnit source;
	public ArrayList<SuperExpr> superExprList = new ArrayList<>();
	public ArrayList<ClassOrInterfaceDeclaration> classOrInterfaceDeclarationList = new ArrayList<>();
	public ArrayList<ClassOrInterfaceType> classOrInterfaceTypeList = new ArrayList<>();

	
	public InheritanceMetrics(CompilationUnit source) {
		this.source = source;
		calculateMetrics();
	}
	
	public void calculateMetrics() {
		createSuperExprList();
		createArrayAccessExprList();
		createArrayCreationExprList();
	};
	
	private void createSuperExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final SuperExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	superExprList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createArrayCreationExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ClassOrInterfaceDeclaration n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	classOrInterfaceDeclarationList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	} 
	
	private void createArrayAccessExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ClassOrInterfaceType n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	classOrInterfaceTypeList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	} 
	
    public String toString() {
    	
    	if (superExprList.isEmpty() && classOrInterfaceDeclarationList.isEmpty() && classOrInterfaceTypeList.isEmpty())
    		return "";
    	else { 	
    		return "INHERITANCE METRICS: \n\n" + 
            		new ListPrinter(superExprList, "Super Expression List: ")
            		+ new ListPrinter(classOrInterfaceDeclarationList, "Class or Interface Declaration list: ")
    				+ new ListPrinter(classOrInterfaceTypeList, "Class or Interface Type List: ");
    		}
          		
    }
}
