package metrics.iterable;

import java.util.ArrayList;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.expr.ArrayAccessExpr;
import com.github.javaparser.ast.expr.ArrayCreationExpr;
import com.github.javaparser.ast.expr.ArrayInitializerExpr;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;

import list.printer.ListPrinter;

public class IterableMetrics {
	CompilationUnit source;
	public ArrayList<ArrayInitializerExpr> arrayInitExprList = new ArrayList<>();
	public ArrayList<ArrayCreationExpr> arrayCreationExprList = new ArrayList<>();
	public ArrayList<ArrayAccessExpr> arrayAccessExprList = new ArrayList<>();

	
	public IterableMetrics(CompilationUnit source) {
		this.source = source;
		calculateMetrics();
	}
	
	public void calculateMetrics() {
		createArrayInitExprList();
		createArrayAccessExprList();
		createArrayCreationExprList();
	};
	
	private void createArrayInitExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ArrayInitializerExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	arrayInitExprList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createArrayCreationExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ArrayCreationExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	arrayCreationExprList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	} 
	
	private void createArrayAccessExprList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ArrayAccessExpr n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	arrayAccessExprList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	} 

    public String toString() {
    	if (arrayInitExprList.isEmpty() &&  arrayCreationExprList.isEmpty() && arrayAccessExprList.isEmpty())
    		return "";
    	else { 	
    		return "ITERATABLE METRICS: \n\n" + 
            		new ListPrinter(arrayInitExprList,"Array Initializer list : ")
            		+ new ListPrinter(arrayCreationExprList, "Array Creation list : ")
            		+ new ListPrinter(arrayAccessExprList, "Array Access list : ");   	
    		}	
    }
}
