package metrics.iterator;

import java.util.ArrayList;
import java.util.List;
import com.github.javaparser.ast.CompilationUnit;
import com.github.javaparser.ast.expr.LambdaExpr;
import com.github.javaparser.ast.stmt.DoStmt;
import com.github.javaparser.ast.stmt.ForStmt;
import com.github.javaparser.ast.stmt.ForeachStmt;
import com.github.javaparser.ast.stmt.WhileStmt;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;

import list.printer.ListPrinter;


public class IteratorMetrics {
	private CompilationUnit source;
	public List<ForStmt> forStatementList = new ArrayList<>();
	public List<WhileStmt> whileStatementList = new ArrayList<>();
	public List<ForeachStmt> foreachStatementList = new ArrayList<>();
	public List<LambdaExpr> lambdaExpressionList = new ArrayList<>();
	public List<DoStmt> doStmtList = new ArrayList<>();
	
	public IteratorMetrics(CompilationUnit source) {
		this.source = source;
		calculateMetrics();
	}
	
	public void calculateMetrics() {

		createForStatementList();
		createForeachStatementList();
		createWhileStatementList();
		createDoStatementList();
		createLambdaExpressionList();
	}

	private void createForStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ForStmt n, final Void arg){
	                //System.out.println("for statement  statement :"+n);
	                forStatementList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createWhileStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final WhileStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
	                whileStatementList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createForeachStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final ForeachStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
	                foreachStatementList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createLambdaExpressionList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final LambdaExpr n, final Void arg){
	                //System.out.println("Lambda statement :"+n);
	                lambdaExpressionList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	}
	
	private void createDoStatementList() {
		source.accept(new VoidVisitorAdapter<Void>(){
			    @Override
	            public void visit(final DoStmt n, final Void arg){
	              //  System.out.println("If statement :"+n);
			    	doStmtList.add(n);
	                super.visit(n, arg);
	            };
	        }, null);
	} 

    public String toString() {
    	if (forStatementList.isEmpty() &&  foreachStatementList.isEmpty() && whileStatementList.isEmpty() && lambdaExpressionList.isEmpty() && doStmtList.isEmpty())
    		return "";
    	else { 	
    		return "ITERATOR METRICS: \n\n" + 
            		new ListPrinter(foreachStatementList,"Foreach iterations: ")
            		+ new ListPrinter(forStatementList, "For iterations: ")
            		+ new ListPrinter(whileStatementList, "While iterations : ")
            		+ new ListPrinter(doStmtList, "Do statements: ")
            		+ new ListPrinter(lambdaExpressionList, "Lambda expression iterations: ");   	
    		} 	
  	
    }
}