package summary.metric.extraction;

import java.util.ArrayList;
import java.util.List;

import com.github.javaparser.ast.expr.Expression;
import com.github.javaparser.ast.stmt.IfStmt;
import com.github.javaparser.ast.stmt.SwitchStmt;

import metrics.collection.MetricsCollection;
import metrics.collection.MetricsCollectionList;


public class ConditionalStatementsSummary {
	public List<Expression> conditionScentenceList = new ArrayList<>();
	public List<String> booleanOperatorList = new ArrayList<>();
	private List<MetricsCollection> metricsCollectionList;
	public int cyclomaticComplexity = 0;
	public int numberOfIfs = 0; 
	public int numberOfElses = 0;
	public int numberOfSwitches = 0;
	public int numberOfOperatorEqual = 0;
	public int numberOfOperatorNotEqual = 0;
	public int numberOfOperatorAnd = 0;
	public int numberOfOperatorOr = 0;
	public int numberOfOperatorLessThan = 0;
	public int numberOfOperatorMoreThan = 0;
	public int numberOfOperatorMoreOrEqual = 0;
	public int numberOfOperatorLessOrEqual = 0; 
	public int numberOfBooleanFunction = 0;

	public ConditionalStatementsSummary(MetricsCollectionList mcl) {
		this.metricsCollectionList = mcl.metricsCollectionList;
		for(MetricsCollection metricsCollection:metricsCollectionList) {
			countNumberOfIfs(metricsCollection);
			countElseStatements(metricsCollection);
			countSwitchStatements(metricsCollection);
			countBooleanOperators(metricsCollection);
			getHighestCyclomaticComplexity(metricsCollection);	
		}
		
	}
	
	private void getHighestCyclomaticComplexity(MetricsCollection metricsCollection) {
		if(metricsCollection.conditionalStatementsMetrics.cyclomaticComplexity>cyclomaticComplexity){
			cyclomaticComplexity = metricsCollection.conditionalStatementsMetrics.cyclomaticComplexity;
		}
	}

	private void countNumberOfIfs(MetricsCollection metricsCollection) {
		numberOfIfs = numberOfIfs + metricsCollection.conditionalStatementsMetrics.ifStmtList.size();
	}
		
	private void countBooleanOperators(MetricsCollection metricsCollection) {
		
		numberOfOperatorEqual += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfEquals;
		numberOfOperatorNotEqual += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfNotEquals;
		numberOfOperatorAnd += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfAnd;
		numberOfOperatorOr += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfOr;
		numberOfOperatorLessThan += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfLessThan;
		numberOfOperatorMoreThan += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfMoreThan;
		numberOfOperatorLessOrEqual += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfLessOrEqual;
		numberOfOperatorMoreOrEqual += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfMoreOrEqual;
		numberOfBooleanFunction += metricsCollection.conditionalStatementsMetrics.conditionalOperatorCounter.numberOfBooleanFunctions;
		
		//Handeling that operator "more than" and "less than" is counted by "less or equal" and "more or equal" as well. 
		numberOfOperatorLessThan = numberOfOperatorLessThan - numberOfOperatorLessOrEqual;
		numberOfOperatorMoreThan = numberOfOperatorMoreThan - numberOfOperatorMoreOrEqual;
	}
	
	private void countElseStatements(MetricsCollection metricsCollection) {
		for(IfStmt ifStatement:metricsCollection.conditionalStatementsMetrics.ifStmtList){
			if (ifStatement.getElseStmt().isPresent()) {
				numberOfElses = numberOfElses + 1; 
			}
		}			
			
	}
	
	private void countSwitchStatements(MetricsCollection metricsCollection) {
		for(SwitchStmt switchStatement:metricsCollection.conditionalStatementsMetrics.switchStmtList){
			numberOfSwitches = numberOfSwitches + 1; 
		}			
	}	
	
	
	
	
	/*
	//Returns try and catch 
	public void findThenStatements() {
		for (MetricsCollection metricsCollection:metricsCollectionList) {
			for(IfStmt ifStatement:metricsCollection.conditionalStatementsMetrics.ifStmtList){
				System.out.println("This is the 'then' statements: "+ifStatement.getThenStmt()); 
			}			
		}	
	}
	*/
	
	private String printIfNotZero (String s, int n) {
		if(n==0) {
			return "";
		}
		else {
			return s + n;
		}
	}
	

	public String toString() {
		return "ConditionalStatementMetrics \n" +
				
				printIfNotZero("\tCyclomatic Complexity: ", cyclomaticComplexity) + 	
				printIfNotZero( "\n\tNumber of elses: ", numberOfElses ) +
				printIfNotZero("\n\tNumber of Ifs: " , numberOfIfs ) +
				printIfNotZero("\n\tNumber of Switches: " , numberOfSwitches ) +
				 "\n\tOperators: \n" + 
				 printIfNotZero("\t\tNumber of Equals (==): " , numberOfOperatorEqual ) +
				 printIfNotZero("\n\t\tNumber of Not Equals (!=): " , numberOfOperatorNotEqual ) +
				 printIfNotZero("\n\t\tNumber of And (&&): " , numberOfOperatorAnd ) +
				 printIfNotZero("\n\t\tNumber of Or (||): " , numberOfOperatorOr ) +
				 printIfNotZero("\n\t\tNumber of Less than (<): " , numberOfOperatorLessThan ) +
				 printIfNotZero("\n\t\tNumber of More than (>): " , numberOfOperatorMoreThan ) +
				 printIfNotZero("\n\t\tNumber of Less or equal (<=): " , numberOfOperatorLessOrEqual ) +
				 printIfNotZero("\n\t\tNumber of More or equal (>=): " , numberOfOperatorMoreOrEqual ) +
				 printIfNotZero("\n\t\tNumber of Boolean function 'fun()': " , numberOfBooleanFunction ) + "\n"
				;
	}
}
