package summary.metric.extraction;


import java.util.List;


import metrics.collection.MetricsCollection;
import metrics.collection.MetricsCollectionList;

public class ExceptionMetricsSummary {
	
	public int numberOfTryStatements = 0;
	public int numberOfThrowStatements = 0;
	public int numberOfCatchStatements =0; 
	public int numberOfIllegalArgumentExceptions =0;
	
	private List<MetricsCollection> metricsCollectionList;

	public ExceptionMetricsSummary(MetricsCollectionList mcl) {
		this.metricsCollectionList = mcl.metricsCollectionList;
		calculateMetricsSummary();
	}
	
	private void calculateMetricsSummary() {
		for (MetricsCollection metricsCollection: metricsCollectionList) {
			countTryStatements(metricsCollection);
			countThrowStatements(metricsCollection);
			countCatchStatements(metricsCollection);
			countArgumentExceptions(metricsCollection);
		}
	}
	
	private void countArgumentExceptions(MetricsCollection metricsCollection) {
		numberOfIllegalArgumentExceptions += metricsCollection.exceptionMetrics.numberOfArgumentExceptions; 
	}
	
	private void countTryStatements(MetricsCollection metricsCollection) {
			numberOfTryStatements += metricsCollection.exceptionMetrics.tryStmtList.size();
	}
	
	private void countThrowStatements(MetricsCollection metricsCollection) {
			numberOfThrowStatements += metricsCollection.exceptionMetrics.throwStmtList.size();
	}
	
	private void countCatchStatements(MetricsCollection metricsCollection) {
			numberOfCatchStatements += metricsCollection.exceptionMetrics.catchStmtList.size();
	}
	
	/*public List<ThrowStmt> throwStmtList = new ArrayList<>();
	public List<CatchClause> catchStmtList = new ArrayList<>();
	 */
	
	private String printIfNotZero (String s, int n) {
		if(n==0) {
			return "";
		}
		else {
			return s + n;
		}
	}
	
	
	public String toString() {
		return "ExceptionMetrics\n" + 
				printIfNotZero("\tNumber of Try statements: " , numberOfTryStatements)+ 
				printIfNotZero("\n\tNumber of Catch statements: " , numberOfCatchStatements )+
				printIfNotZero("\n\tNumber of Throw statements: " , numberOfThrowStatements  )+
				"\n\tExceptions: \n"+
				printIfNotZero("\t\tNumber of IllegalArgumentExceptions: " , numberOfIllegalArgumentExceptions)
				+"\n\n"
				;
				
	}
}
