package summary.metric.extraction;

import java.util.List;
import metrics.collection.MetricsCollection;
import metrics.collection.MetricsCollectionList;

public class GeneralMetricsSummary {

	private List<MetricsCollection> metricsCollectionList;
	public int linesTotal = 0;
	public int methodCallsTotal = 0;
	public int variableDeclarationsTotal = 0;
	
	//FieldDeclarations
	public int numberOfPrivate = 0;
	public int numberOfPublic = 0;
	public int numberOfProtected = 0;
	
	
	//UnaryOperators
	public int numberOfMultiply = 0; 
	public int numberOfDivide = 0;
	public int numberOfAddition = 0;
	public int numberOfSubtraction = 0;
	public int numberOfPlusOne = 0;
	public int numberOfMinusOne = 0;
	public int numberOfBooleanInverter = 0;
	public int numberOfRemains = 0;
	
	//Weighted methods per class
	public int totalNumberOfMethods = 0;
	public int largestMethodSize = 0;
	public int hightestMethodComplexity = 0;
	 
	public GeneralMetricsSummary(MetricsCollectionList mcl) {
		this.metricsCollectionList = mcl.metricsCollectionList;
		calculateMetrics();
	}
	
	private void calculateMetrics() {
		countTotalCodeLines();
		
	}
	
	private void countTotalCodeLines() {
		for (MetricsCollection metricsCollection:metricsCollectionList) {
			linesTotal = linesTotal + metricsCollection.generalMetrics.linesCount;
			methodCallsTotal = methodCallsTotal + metricsCollection.generalMetrics.methodCallsList.size();
			variableDeclarationsTotal = variableDeclarationsTotal + metricsCollection.generalMetrics.variableDeclarationExprList.size();
			countUnaryOperators(metricsCollection);
			countFieldDeclarations(metricsCollection);
			getWeightedMethodsPerClass(metricsCollection);
		}
	}
	
	private void countUnaryOperators(MetricsCollection metricsCollection) {
		this.numberOfMultiply += metricsCollection.generalMetrics.unaryOperators.numberOfMultiply; 
		this.numberOfDivide += metricsCollection.generalMetrics.unaryOperators.numberOfDivide;
		this.numberOfAddition += metricsCollection.generalMetrics.unaryOperators.numberOfAddition;
		this.numberOfSubtraction += metricsCollection.generalMetrics.unaryOperators.numberOfSubtraction;
		this.numberOfPlusOne += metricsCollection.generalMetrics.unaryOperators.numberOfPlusOne;
		this.numberOfMinusOne += metricsCollection.generalMetrics.unaryOperators.numberOfMinusOne;
		this.numberOfBooleanInverter += metricsCollection.generalMetrics.unaryOperators.numberOfBooleanInverter;
		this.numberOfRemains += metricsCollection.generalMetrics.unaryOperators.numberOfRemains;
	}
	
	private void countFieldDeclarations(MetricsCollection metricsCollection) {
		this.numberOfPrivate += metricsCollection.generalMetrics.fieldDeclarations.numberOfPrivate; 
		this.numberOfPublic += metricsCollection.generalMetrics.fieldDeclarations.numberOfPublic;
		this.numberOfProtected += metricsCollection.generalMetrics.fieldDeclarations.numberOfProtected;

	}
	
	private void getWeightedMethodsPerClass(MetricsCollection metricsCollection) {
		this.totalNumberOfMethods += metricsCollection.generalMetrics.numberOfMethods; 
		if(metricsCollection.generalMetrics.largestMethod>largestMethodSize)
			this.largestMethodSize = metricsCollection.generalMetrics.largestMethod;
		if(metricsCollection.generalMetrics.hightestMethodComplexity>hightestMethodComplexity)
			this.hightestMethodComplexity = metricsCollection.generalMetrics.hightestMethodComplexity;
	}

	private String printIfNotZero (String s, int n) {
		if(n==0) {
			return "";
		}
		else {
			return s + n;
		}
	}
	
	
	public String toString() {
		return "GeneralMetrics:\n"+
				"\n\tTotal number of code lines: " + linesTotal +"\n"+
				"\n\tTotal number of variables declared: " + variableDeclarationsTotal +"\n"+
				"\n\tTotal number of method calls: " +methodCallsTotal+"\n"
				+"\tNumber of unaryOperators\n"+
				printIfNotZero ("\t\tMultiply: ", numberOfMultiply) + 
				printIfNotZero ("\n\t\tDivide: ", numberOfDivide) +
				printIfNotZero ("\n\t\tAddition: ", numberOfAddition) +
				printIfNotZero ("\n\t\tSubtraction: ", numberOfSubtraction ) +
				printIfNotZero ("\n\t\tPlusOne (++): ", numberOfPlusOne ) +
				printIfNotZero ("\n\t\tMinusOne (--): ", numberOfMinusOne ) +
				printIfNotZero ("\n\t\tBooleanInverter (!): ", numberOfBooleanInverter ) +
				printIfNotZero ("\n\t\tRemains (%): ", numberOfRemains ) +
				"\n\n\tWheighted Methods per Class: \n"+
				printIfNotZero ("\n\t\tNumberOfMethods: ", totalNumberOfMethods ) +
				printIfNotZero ("\n\t\tLargestMethodSize: ", largestMethodSize ) +
				printIfNotZero ("\n\t\tHighestCyclomaticComplexity: ", hightestMethodComplexity ) + "\n";
	}
}

